"""Add receiving, shipping and inventory_transaction related tables

Revision ID: 1b3411739b82
Revises: 7b2d863b105
Create Date: 2015-07-03 21:45:51.072263

"""

# revision identifiers, used by Alembic.
revision = '1b3411739b82'
down_revision = '7b2d863b105'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('inventory_transaction',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('date', sa.DateTime(), nullable=False),
    sa.Column('type_id', sa.Integer(), nullable=False),
    sa.Column('remark', sa.Text(), nullable=True),
    sa.ForeignKeyConstraint(['type_id'], ['enum_values.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('inventory_transaction_line',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('in_transit_quantity', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=True),
    sa.Column('quantity', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=True),
    sa.Column('product_id', sa.Integer(), nullable=False),
    sa.Column('price', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
    sa.Column('remark', sa.Text(), nullable=True),
    sa.Column('inventory_transaction_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['inventory_transaction_id'], ['inventory_transaction.id'], ),
    sa.ForeignKeyConstraint(['product_id'], ['product.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('receiving',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('date', sa.DateTime(), nullable=False),
    sa.Column('remark', sa.Text(), nullable=True),
    sa.Column('status_id', sa.Integer(), nullable=False),
    sa.Column('purchase_order_id', sa.Integer(), nullable=False),
    sa.Column('inventory_transaction_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['purchase_order_id'], ['purchase_order.id'], ),
    sa.ForeignKeyConstraint(['inventory_transaction_id'], ['inventory_transaction.id'], ),
    sa.ForeignKeyConstraint(['status_id'], ['enum_values.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('receiving_line',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('quantity', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
    sa.Column('price', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
    sa.Column('product_id', sa.Integer(), nullable=False),
    sa.Column('receiving_id', sa.Integer(), nullable=False),
    sa.Column('purchase_order_line_id', sa.Integer(), nullable=False),
    sa.Column('inventory_transaction_line_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['product_id'], ['product.id'], ),
    sa.ForeignKeyConstraint(['purchase_order_line_id'], ['purchase_order_line.id'], ),
    sa.ForeignKeyConstraint(['inventory_transaction_line_id'], ['inventory_transaction_line.id'], ),
    sa.ForeignKeyConstraint(['receiving_id'], ['receiving.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('shipping',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('date', sa.DateTime(), nullable=False),
    sa.Column('remark', sa.Text(), nullable=True),
    sa.Column('status_id', sa.Integer(), nullable=False),
    sa.Column('sales_order_id', sa.Integer(), nullable=False),
    sa.Column('inventory_transaction_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['inventory_transaction_id'], ['inventory_transaction.id'], ),
    sa.ForeignKeyConstraint(['sales_order_id'], ['sales_order.id'], ),
    sa.ForeignKeyConstraint(['status_id'], ['enum_values.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('shipping_line',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('quantity', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
    sa.Column('price', sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
    sa.Column('product_id', sa.Integer(), nullable=False),
    sa.Column('shipping_id', sa.Integer(), nullable=False),
    sa.Column('sales_order_line_id', sa.Integer(), nullable=False),
    sa.Column('inventory_transaction_line_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['inventory_transaction_line_id'], ['inventory_transaction_line.id'], ),
    sa.ForeignKeyConstraint(['sales_order_line_id'], ['sales_order_line.id'], ),
    sa.ForeignKeyConstraint(['product_id'], ['product.id'], ),
    sa.ForeignKeyConstraint(['shipping_id'], ['shipping.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('shipping_line')
    op.drop_table('shipping')
    op.drop_table('receiving_line')
    op.drop_table('receiving')
    op.drop_table('inventory_transaction_line')
    op.drop_table('inventory_transaction')
    ### end Alembic commands ###
