""" Add create date field to supplier model, make role.is_system and shipping.type_id not null.

Revision ID: e1f806a716b9
Revises: 052340beb7b5
Create Date: 2017-05-25 08:12:35.839903

"""

# revision identifiers, used by Alembic.
revision = 'e1f806a716b9'
down_revision = '052340beb7b5'

from alembic import op
import sqlalchemy as sa
from sqlalchemy import func 
from sqlalchemy.sql import text
from datetime import datetime


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('role', 'is_system', existing_type=sa.BOOLEAN(),
               nullable=False, existing_server_default=sa.text(u'true'))
    op.alter_column('shipping', 'type_id', existing_type=sa.INTEGER(), nullable=False)
    op.add_column('supplier', sa.Column('create_date', sa.DateTime(), nullable=True))

    results = op.get_bind().execute(text("""
    select sup.id, min(so.order_date) from sales_order so, supplier sup,
    sales_order_line sol, product p where so.id = sol.sales_order_id and
    sol.product_id = p.id and p.supplier_id = sup.id group by sup.id;
    """)).fetchall()
    for r in results:
        sup_id = r[0]
        so_date = r[1]
        op.get_bind().execute(text("update supplier set create_date = '{0}' where id={1}".format(so_date, sup_id)))

    results = op.get_bind().execute(text("""
    select sup.id, min(po.order_date) from purchase_order po, supplier sup
    where po.supplier_id = sup.id group by sup.id
    """)).fetchall()
    for r in results:
        sup_id = r[0]
        po_date = r[1]
        op.get_bind().execute(text("update supplier set create_date = '{0}' where id={1} and create_date is null".format(po_date, sup_id)))

    op.get_bind().execute(text("update supplier set create_date = '{0}' where create_date is null".format(datetime.now())))
    op.alter_column('supplier', 'create_date', existing_type=sa.DateTime(), nullable=False)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('supplier', 'create_date')
    op.alter_column('shipping', 'type_id',
               existing_type=sa.INTEGER(),
               nullable=True)
    op.alter_column('role', 'is_system',
               existing_type=sa.BOOLEAN(),
               nullable=True,
               existing_server_default=sa.text(u'true'))
    # ### end Alembic commands ###
